/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.milaifontanals;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintStream;
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.Objects;

/**
 *
 * @author Usuari
 */
public class Electrodomestic implements Comparable<Electrodomestic>, Serializable {

    private int id;         // Generació automàtica a partir de qElectrodomesticsCreats
    private String nom;     // Cadena obligatòria amb longitud mínima 4
    private String marca;    // Cadena obligatòria no buida
    private double preu;     // Real obligatori i positiu
    private Byte garantia;     // Enter optatiu i en tot cas positiu
    private GregorianCalendar dataPatent;   // Opcional
    private GregorianCalendar momentCreacio = new GregorianCalendar();
    private GregorianCalendar momentLastModificacio;
    private static Float dteVigent;    // Real optatiu entre 0 i 100 
    // Estàtic => Vàlid per a tots els electrodomèstics

    protected static int qElectrodomesticsCreats;      // Comptador intern - Inicialment a zero
    private static int qElectrodomesticsVius;        // Comptador intern - Inicialment a zero

    public Electrodomestic(String nom, String marca, double preu, Byte garantia,
            GregorianCalendar dataPatent) {
        setNom(nom);
        setMarca(marca);
        setPreu(preu);
        setGarantia(garantia);
        setDataPatent(dataPatent);
        momentLastModificacio = null;
        // Posem l'assignació d'id i l'increment de qElectrodomesticsCreats quan 
        // estemn segurs de que l'objecte es pot crear (que no peta)
        qElectrodomesticsCreats++;
        qElectrodomesticsVius++;
        id = qElectrodomesticsCreats;

    }

    public Electrodomestic(String nom, String marca, double preu) {
        this(nom, marca, preu, null, null);
    }

    public final String getNom() {
        return nom;
    }

    public void setNom(String nom) {
        if (nom == null || nom.length() < 4) {
            throw new RuntimeException("Nom ha de ser obligatori i de longitud mínima 4");
        }
        this.nom = nom;
        this.momentLastModificacio = new GregorianCalendar();
    }

    public final String getMarca() {
        return marca;
    }

    public void setMarca(String marca) {
        if (marca == null) {
            throw new RuntimeException("Marca obligatòria");
        }
        if (marca.length() == 0) {
            throw new RuntimeException("Marca no buida");
        }
        this.marca = marca;
        this.momentLastModificacio = new GregorianCalendar();
    }

    public final double getPreu() {
        return preu;
    }

    public void setPreu(double preu) {
        if (preu < 0) {
            throw new RuntimeException("El preu ha de ser positiu");
        }
        this.preu = preu;
        this.momentLastModificacio = new GregorianCalendar();
    }

    public static Float getDteVigent() {
        return dteVigent;
    }

    public static void setDteVigent(Float dteVigent) {
        if (dteVigent != null && (dteVigent < 0 || dteVigent > 100)) {
            throw new RuntimeException("Dte. vigent optatiu entre 0 i 100");
        }
        Electrodomestic.dteVigent = dteVigent;
    }

    public final Byte getGarantia() {
        return garantia;
    }

    public void setGarantia(Byte garantia) {
        if (garantia != null && garantia <= 0) {
            throw new RuntimeException("La garantia, si existeix, ha de ser positiva");
        }
        this.garantia = garantia;
        this.momentLastModificacio = new GregorianCalendar();
    }

    public final GregorianCalendar getDataPatent() {
        return dataPatent;
    }

    public void setDataPatent(GregorianCalendar dataPatent) {
        if (dataPatent == null) {
            this.dataPatent = null;
        } else {
            this.dataPatent = (GregorianCalendar) dataPatent.clone();
        }
        this.momentLastModificacio = new GregorianCalendar();
    }

    public final int getId() {
        return id;
    }

    public final static int getqElectrodomesticsCreats() {
        return qElectrodomesticsCreats;
    }

    public final static int getqElectrodomesticsVius() {
        return qElectrodomesticsVius;
    }

    public final GregorianCalendar getMomentCreacio() {
        return momentCreacio;
    }

    public final GregorianCalendar getMomentLastModificacio() {
        return momentLastModificacio;
    }

//  Els següents mètodes no "poden" existir, doncs no volem "de moment" que ningú
//  pugui canviar el Id d'un electrodomèstic o el comptador.
//  En ocasions hi ha eines "Java" que obliguen l'existència de tots setters...
//  En tal cas, els podríem tenir "private"
//    public void setId(int id) {
//        this.id = id;
//    }
//    public static void setqElectrodomestics(int qElectrodomesticsCreats) {
//        Electrodomestic.qElectrodomesticsCreats = qElectrodomesticsCreats;
//    }
    @Override
    public String toString() {
        String resultat = "Electrodomestic:" + "id=" + id + ", nom=" + nom + ", marca=" + marca + ", preu=" + preu + ", garantia=";
        if (garantia == null) {
            resultat += "???";
        } else {
            resultat += garantia;
        }
        resultat += ", dataPatent=";
        if (dataPatent == null) {
            resultat += "???";
        } else {
            SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy");
            resultat += sdf.format(dataPatent.getTime());
        }
        SimpleDateFormat moment = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");
        resultat += ", momentCreacio=" + moment.format(this.momentCreacio.getTime());
        if (this.momentLastModificacio != null) {
            resultat += ", darreraModificacio=" + moment.format(this.momentLastModificacio.getTime());
        }
        return resultat;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 79 * hash + Objects.hashCode(this.nom);
        hash = 79 * hash + Objects.hashCode(this.marca);
        return hash;
    }

    @Override
    /**
     * Considerarem que dos electrodomèstics són iguals si tenen igual nom i
     * marca (case insensitive)
     */
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        Electrodomestic aux = (Electrodomestic) obj;
        return this.nom.equalsIgnoreCase(aux.nom) && this.marca.equalsIgnoreCase(aux.marca);
    }

//    @Override
//    public abstract Object clone() throws CloneNotSupportedException;
    @Override
    protected void finalize() throws Throwable {
        qElectrodomesticsVius--;
        System.out.println("S'està morint l'electrodomestic " + id);
        super.finalize(); //To change body of generated methods, choose Tools | Templates.
    }

    public void visualitzar(PrintStream obj) {
        obj.print(this);
    }

    public void visualitzar(PrintStream obj, boolean saltLinia) {
        if (saltLinia) {
            obj.println(this);
        } else {
            obj.print(this);
        }
    }

    @Override
    public int compareTo(Electrodomestic o) {
        int resultat = this.nom.compareToIgnoreCase(o.nom);
        if (resultat == 0) {
            return this.marca.compareToIgnoreCase(o.marca);
        } else {
            return resultat;
        }
    }

    private void writeObject(ObjectOutputStream out)
            throws IOException {
        out.defaultWriteObject();  // Serialització "automàtica"
        out.writeFloat(Electrodomestic.dteVigent);
    }

    private void readObject(ObjectInputStream in)
            throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        // La següent instrucció recupera SEMPRE el valor
        // i el substitueix a la classe.
        // Electrodomestic.dteVigent = in.readFloat();
        // En cas que no interessi, es pot fer quelcom 
        // similar a:
        Float dteVigentRebut = in.readFloat();
        // Posar la condició convenient per 
        // emplenar o no dteVigent. Aquí emplenem:
        Electrodomestic.dteVigent = dteVigentRebut;
    }

}
