/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.milaifontanals;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintStream;
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.Objects;

/**
 *
 * @author Usuari
 */
public class Electrodomestic implements Comparable<Electrodomestic>, Serializable {

    private int id;         // Obligatori i estrictament positiu
    private String nom;     // Cadena obligatòria amb longitud mínima 4
    private String marca;    // Cadena obligatòria no buida
    private double preu;     // Real obligatori i positiu
    private Byte garantia;     // Enter optatiu i en tot cas positiu
    private GregorianCalendar dataPatent;   // Opcional
    private static Float dteVigent;    // Real optatiu entre 0 i 100 
    // Estàtic => Vàlid per a tots els electrodomèstics

    public Electrodomestic(int id, String nom, String marca, double preu, Byte garantia,
            GregorianCalendar dataPatent) {
        setId(id);
        setNom(nom);
        setMarca(marca);
        setPreu(preu);
        setGarantia(garantia);
        setDataPatent(dataPatent);

    }

    public Electrodomestic(int id, String nom, String marca, double preu) {
        this(id, nom, marca, preu, null, null);
    }

    public int getId() {
        return id;
    }

    public void setId(int id) {
        if (id<=0) {
            throw new RuntimeException("L'identificador ha de ser estrictament positiu");
        }
        this.id = id;
    }

    
    public final String getNom() {
        return nom;
    }

    public void setNom(String nom) {
        if (nom == null || nom.length() < 4) {
            throw new RuntimeException("Nom ha de ser obligatori i de longitud mínima 4");
        }
        this.nom = nom;
    }

    public final String getMarca() {
        return marca;
    }

    public void setMarca(String marca) {
        if (marca == null) {
            throw new RuntimeException("Marca obligatòria");
        }
        if (marca.length() == 0) {
            throw new RuntimeException("Marca no buida");
        }
        this.marca = marca;
    }

    public final double getPreu() {
        return preu;
    }

    public void setPreu(double preu) {
        if (preu < 0) {
            throw new RuntimeException("El preu ha de ser positiu");
        }
        this.preu = preu;
    }

    public static Float getDteVigent() {
        return dteVigent;
    }

    public static void setDteVigent(Float dteVigent) {
        if (dteVigent != null && (dteVigent < 0 || dteVigent > 100)) {
            throw new RuntimeException("Dte. vigent optatiu entre 0 i 100");
        }
        Electrodomestic.dteVigent = dteVigent;
    }

    public final Byte getGarantia() {
        return garantia;
    }

    public void setGarantia(Byte garantia) {
        if (garantia != null && garantia <= 0) {
            throw new RuntimeException("La garantia, si existeix, ha de ser positiva");
        }
        this.garantia = garantia;
    }

    public final GregorianCalendar getDataPatent() {
        return dataPatent;
    }

    public void setDataPatent(GregorianCalendar dataPatent) {
        if (dataPatent == null) {
            this.dataPatent = null;
        } else {
            this.dataPatent = (GregorianCalendar) dataPatent.clone();
        }
    }

    @Override
    public String toString() {
        String resultat = "Electrodomestic:" + "id=" + id + ", nom=" + nom + ", marca=" + marca + ", preu=" + preu + ", garantia=";
        if (garantia == null) {
            resultat += "???";
        } else {
            resultat += garantia;
        }
        resultat += ", dataPatent=";
        if (dataPatent == null) {
            resultat += "???";
        } else {
            SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy");
            resultat += sdf.format(dataPatent.getTime());
        }
        return resultat;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 79 * hash + Objects.hashCode(this.nom);
        hash = 79 * hash + Objects.hashCode(this.marca);
        return hash;
    }

    @Override
    /**
     * Considerarem que dos electrodomèstics són iguals si tenen igual nom i
     * marca (case insensitive)
     */
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        Electrodomestic aux = (Electrodomestic) obj;
        return this.nom.equalsIgnoreCase(aux.nom) && this.marca.equalsIgnoreCase(aux.marca);
    }

    public void visualitzar(PrintStream obj) {
        obj.print(this);
    }

    public void visualitzar(PrintStream obj, boolean saltLinia) {
        if (saltLinia) {
            obj.println(this);
        } else {
            obj.print(this);
        }
    }

    @Override
    public int compareTo(Electrodomestic o) {
        int resultat = this.nom.compareToIgnoreCase(o.nom);
        if (resultat == 0) {
            return this.marca.compareToIgnoreCase(o.marca);
        } else {
            return resultat;
        }
    }

    private void writeObject(ObjectOutputStream out)
            throws IOException {

    }

    private void readObject(ObjectInputStream in)
            throws IOException, ClassNotFoundException {

    }

}
