/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.milaifontanals;

import jakarta.xml.bind.annotation.XmlElement;
import jakarta.xml.bind.annotation.XmlRootElement;
import jakarta.xml.bind.annotation.XmlTransient;
import jakarta.xml.bind.annotation.XmlType;
import jakarta.xml.bind.annotation.adapters.XmlJavaTypeAdapter;
import org.milaifontanals.adapters.DecimalSeparatorCommaAdapter;

/**
 *
 * @author Usuari
 */
@XmlRootElement(name = "whiteRange")
@XmlType(propOrder = {"alturaByJaxb", "ampladaByJaxb", "fondariaByJaxb"})
public final class GammaBlanca extends Electrodomestic {

    private double mides[] = new double[3];     // Totes a zero
    // Contingut de les cel·les:
    // 0: altura
    // 1: amplada
    // 2: fondària

    // Mètode afegit per marcatge JAXB
    protected GammaBlanca() {

    }

    public GammaBlanca(int id, String nom, String marca, double preu, double[] mides) {
        super(id, nom, marca, preu);
        setMides(mides);
    }

    public GammaBlanca(int id, String nom, String marca, double preu,
            double altura, double amplada, double fondaria) {
        super(id, nom, marca, preu);
        double mides[] = {altura, amplada, fondaria};
        setMides(mides);
    }

    @XmlTransient
    public double[] getMides() {
        return mides;
    }

    public void setMides(double[] mides) {
        if (mides.length != 3) {
            throw new RuntimeException("Per assignar mides cal passar una taula de 3 cel·les");
        }
        if (mides[0] <= 0 || mides[1] <= 0 || mides[2] <= 0) {
            throw new RuntimeException("Les mides d'un electrodomèstic han de ser estrictament positives");
        }
//        for (int i = 0; i <= 2; i++) {
//            this.mides[i] = mides[i];
//        }
//      Equivalent:
        System.arraycopy(mides, 0, this.mides, 0, 3);
    }

    /* Per sentit comú, getter i setter per altura, amplada, fondaria */
    public void setAltura(double altura) {
        if (altura <= 0) {
            throw new RuntimeException("Altura ha de ser estrictament positiva");
        }
        mides[0] = altura;
    }

    public void setAmplada(double amplada) {
        if (amplada <= 0) {
            throw new RuntimeException("Ampladaha de ser estrictament positiva");
        }
        mides[1] = amplada;
    }

    public void setFondaria(double fondaria) {
        if (fondaria <= 0) {
            throw new RuntimeException("Fondària ha de ser estrictament positiva");
        }
        mides[2] = fondaria;
    }

    @XmlTransient
    public double getAltura() {
        return mides[0];
    }

    @XmlTransient
    public double getAmplada() {
        return mides[1];
    }

    @XmlTransient
    public double getFondaria() {
        return mides[2];
    }

    @XmlElement(name = "height", required = true)
    @XmlJavaTypeAdapter(DecimalSeparatorCommaAdapter.class)
    private Number getAlturaByJaxb() {
        return getAltura();
    }

    private void setAlturaByJaxb(Number x) {
        setAltura(x.doubleValue());
    }

    @XmlElement(name = "width", required = true)
    @XmlJavaTypeAdapter(DecimalSeparatorCommaAdapter.class)
    private Number getAmpladaByJaxb() {
        return getAmplada();
    }

    private void setAmpladaByJaxb(Number x) {
        setAmplada(x.doubleValue());
    }

    @XmlElement(name = "depth", required = true)
    @XmlJavaTypeAdapter(DecimalSeparatorCommaAdapter.class)
    private Number getFondariaByJaxb() {
        return getFondaria();
    }

    private void setFondariaByJaxb(Number x) {
        setFondaria(x.doubleValue());
    }

    @Override
    public String toString() {
        // Per accedir a camps privats de la classe base, cal usar els getter:
//        return "Electrodomestic Gamma Blanca {" + "nom=" + getNom() + ", altura=" + mides[0] + ", amplada=" + mides[1] + ", fondaria=" + mides[2] + '}';
        return "Gamma Blanca " + super.toString() + ", altura=" + mides[0] + ", amplada=" + mides[1] + ", fondaria=" + mides[2] + '}';
    }

    @Override
    public Object clone() throws CloneNotSupportedException {
        GammaBlanca gb = new GammaBlanca(this.getId(), this.getNom(), this.getMarca(), this.getPreu(),
                this.mides);
        gb.setGarantia(this.getGarantia());
        gb.setDataPatent(this.getDataPatent());
        return gb;
    }

}
