/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.milaifontanals;

import java.io.PrintStream;
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.Objects;
import jakarta.xml.bind.annotation.XmlAttribute;
import jakarta.xml.bind.annotation.XmlElement;
import jakarta.xml.bind.annotation.XmlTransient;
import jakarta.xml.bind.annotation.XmlType;
import jakarta.xml.bind.annotation.adapters.XmlJavaTypeAdapter;
import org.milaifontanals.adapters.DecimalSeparatorCommaAdapter;
import org.milaifontanals.adapters.GregorianCalendarAdapter1;

/**
 *
 * @author Usuari
 */
//@XmlRootElement(name="homeAppliance")   // És abstracta - Ho indicarem a les classes no abstractes
@XmlType(propOrder = {"nom", "marca", "preuByJaxb", "garantia", "dataPatent"})
public abstract class Electrodomestic implements Cloneable, Comparable<Electrodomestic>, Serializable {

    private int id;         // Obligatori i estrictament positiu
    private String nom;     // Cadena obligatòria amb longitud mínima 4
    private String marca;     // Cadena obligatòria no buida
    private double preu;     // Real obligatori i positiu
    private Byte garantia;     // Enter optatiu i en tot cas positiu
    private GregorianCalendar dataPatent;   // Opcional
    private static float dteVigent;      // Real obligatori entre 0 (0%) i 1 (100%)

    // Mètode afegit per marcatge JAXB
    protected Electrodomestic() {

    }

    public Electrodomestic(int id, String nom, String marca, double preu, Byte garantia,
            GregorianCalendar dataPatent) {
        this.setId(id);
        this.setNom(nom);
        this.setMarca(marca);
        setPreu(preu);
        setGarantia(garantia);
        setDataPatent(dataPatent);
    }

    public Electrodomestic(int id, String nom, String marca, double preu) {
        this(id, nom, marca, preu, null, null);
    }

    @XmlTransient
    public final int getId() {
        return id;
    }

    public void setId(int id) {
        if (id <= 0) {
            throw new RuntimeException("L'identificador ha de ser estrictament positiu");
        }
        this.id = id;
    }

    // Mètodes getIdByJaxb i setIdByJaxb afegits per marcatge JAXB
    @XmlAttribute(name = "code", required = true)
    private String getIdByJaxb() {
        return "HA-" + id;
    }

    private void setIdByJaxb(String valor) {
        setId(Integer.parseInt(valor.substring(3)));
    }

    @XmlElement(name = "name", required = true)
    public final String getNom() {
        return nom;
    }

    public void setNom(String nom) {
        if (nom == null || nom.length() < 4) {
            throw new RuntimeException("Nom ha de ser obligatori i de longitud mínima 4");
        }
        this.nom = nom;
    }

    @XmlElement(name = "brand", required = true)
    public final String getMarca() {
        return marca;
    }

    public void setMarca(String marca) {
        if (marca == null) {
            throw new RuntimeException("Marca obligatòria");
        }
        if (marca.length() == 0) {
            throw new RuntimeException("Marca no buida");
        }
        this.marca = marca;
    }

    @XmlTransient
    public final double getPreu() {
        return preu;
    }

    public void setPreu(double preu) {
        if (preu < 0) {
            throw new RuntimeException("El preu ha de ser positiu");
        }
        this.preu = preu;
    }

    // Mètodes getPreuByJaxb i setPreuByJaxb per poder usar classe Adapter
    // doncs les classes adapter no es poden usar amb tipus primitius
    @XmlElement(name = "price", required = false)
    @XmlJavaTypeAdapter(DecimalSeparatorCommaAdapter.class)
    private Number getPreuByJaxb() {
        return preu;
    }

    private void setPreuByJaxb(Number x) {
        setPreu(x.doubleValue());
    }

    @XmlElement(name = "warranty", required = false)
    public final Byte getGarantia() {
        return garantia;
    }

    public void setGarantia(Byte garantia) {
        if (garantia != null && garantia <= 0) {
            throw new RuntimeException("La garantia, si existeix, ha de ser positiva");
        }
        this.garantia = garantia;
    }

    @XmlElement(name = "patentDate", required = false)
    @XmlJavaTypeAdapter(GregorianCalendarAdapter1.class)
    public final GregorianCalendar getDataPatent() {
        return dataPatent;
    }

    public void setDataPatent(GregorianCalendar dataPatent) {
        if (dataPatent == null) {
            this.dataPatent = null;
        } else {
            this.dataPatent = (GregorianCalendar) dataPatent.clone();
        }
    }

    public static float getDteVigent() {
        return dteVigent;
    }

    public static void setDteVigent(float dteVigent) {
        if (dteVigent < 0 || dteVigent > 1) {
            throw new RuntimeException("Dte. vigent ha d'estar entre 0 (0%) i 1(100%)");
        }
        Electrodomestic.dteVigent = dteVigent;
    }

    @Override
    public String toString() {
        String resultat = "Electrodomestic:" + "id=" + id + ", nom=" + nom + ", marca=" + marca + ", preu=" + preu + ", garantia=";
        if (garantia == null) {
            resultat += "???";
        } else {
            resultat += garantia;
        }
        resultat += ", dataPatent=";
        if (dataPatent == null) {
            resultat += "???";
        } else {
            SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy");
            resultat += sdf.format(dataPatent.getTime());
        }
        return resultat;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 79 * hash + Objects.hashCode(this.nom);
        hash = 79 * hash + Objects.hashCode(this.marca);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        Electrodomestic aux = (Electrodomestic) obj;
        return this.nom.equals(aux.nom) && this.marca.equals(aux.marca);
    }

    @Override
    public abstract Object clone() throws CloneNotSupportedException;

    public void visualitzar(PrintStream obj) {
        obj.print(this);
    }

    public void visualitzar(PrintStream obj, boolean saltLinia) {
        if (saltLinia) {
            obj.println(this);
        } else {
            obj.print(this);
        }
    }

    @Override
    public int compareTo(Electrodomestic o) {
        int resultat = this.nom.compareTo(o.nom);
        if (resultat == 0) {
            return this.marca.compareTo(o.marca);
        } else {
            return resultat;
        }
    }

}
