/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.milaifontanals;

import java.util.Arrays;
import java.util.GregorianCalendar;
import java.util.List;
import jakarta.xml.bind.annotation.XmlElement;
import jakarta.xml.bind.annotation.XmlRootElement;
import jakarta.xml.bind.annotation.XmlTransient;
import jakarta.xml.bind.annotation.XmlType;

/**
 *
 * @author Usuari
 */
@XmlRootElement(name = "homeAppliance")
public final class GammaGris extends Electrodomestic {

    private String model;
    /* Obligatori no buit */
    private String efEnergia;
    /* Eficiencia energètica - optatiu - Valors A+++,A++,A+,A,B,C,D,E) */
    private Integer polsades;
    /* Optatiu */
    
    // Mètode afegit per marcatge JAXB
    protected GammaGris() {

    }

    public GammaGris(int id, String nom, String marca, double preu, String model) {
        super(id, nom, marca, preu);
        setModel(model);
    }

    public GammaGris(int id, String nom, String marca, double preu, Byte garantia, GregorianCalendar dataPatent,
            String model, String efEnergia, Integer polsades) {
        super(id, nom, marca, preu, garantia, dataPatent);
        setModel(model);
        setEfEnergia(efEnergia);
        setPolsades(polsades);
    }

    @XmlTransient
    public String getModel() {
        return model;
    }

    public void setModel(String model) {
        if (model == null || model.length() == 0) {
            throw new RuntimeException("Model obligatori i no buit");
        }
        this.model = model;
    }

    @XmlTransient
    public String getEfEnergia() {
        return efEnergia;
    }

    public void setEfEnergia(String efEnergia) {
        List<String> llista = Arrays.asList("A+++", "A++", "A+", "A", "B", "C", "D", "E");
        if (efEnergia != null) {
            efEnergia = efEnergia.toUpperCase();
            if (!llista.contains(efEnergia)) {
                throw new RuntimeException("Valor per Ef. Energètica ha de ser A+++, A++, A+, A, B, C, D, E");
            }
        }
        this.efEnergia = efEnergia;
    }

    @XmlTransient
    public Integer getPolsades() {
        return polsades;
    }

    public void setPolsades(Integer polsades) {
        if (polsades != null && polsades <= 0) {
            throw new RuntimeException("Polsades optatiu o estrictament positiu");
        }
        this.polsades = polsades;
    }

    @Override
    public String toString() {
        // Per accedir a camps privats de la classe base, cal usar els getter:
        // return "Electrodomestic Gamma Gris {" + "nom=" + getNom() + ", model=" + model + ", efEnergia=" + efEnergia + ", polsades=" + polsades + '}';
        return "Gamma Gris " + super.toString() + ", model=" + model + ", efEnergia=" + efEnergia + ", polsades=" + polsades + '}';
    }

    @Override
    public Object clone() throws CloneNotSupportedException {
        GammaGris g = new GammaGris(this.getId(), this.getNom(), this.getMarca(), this.getPreu(), this.getGarantia(),
                this.getDataPatent(), model, efEnergia, polsades);
        return g;
    }

    // Camp afegit per marcatge JAXB
    private GammaGrisAuxPerJaxb aux;

    @XmlElement(name="grayRange", required=true)
    public GammaGrisAuxPerJaxb getAux() {
        GammaGrisAuxPerJaxb aux = new GammaGrisAuxPerJaxb();
        aux.model = this.model;
        aux.efEnergia = this.efEnergia;
        aux.polsades = this.polsades;
        return aux;
    }

    public void setAux(GammaGrisAuxPerJaxb aux) {
        this.setModel(aux.model);
        this.setEfEnergia(aux.efEnergia);
        this.setPolsades(aux.polsades);
    }

    
    
}

@XmlType(propOrder = {"model", "efEnergia", "polsades"})
class GammaGrisAuxPerJaxb {

    @XmlElement(required = true)
    String model;
    @XmlElement(name = "energyEfficiency", required = false)
    String efEnergia;
    @XmlElement(name = "inches", required = false)
    Integer polsades;
}
