/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.milaifontanals;

import java.io.File;
import java.io.IOException;
import java.util.List;
import org.apache.commons.io.FileUtils;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.jdom2.input.sax.XMLReaders;

/**
 *
 * @author Professor
 */
public class CalculPoblacioEuropaV1 {

    public static void main(String args[]) {
        SAXBuilder builder = null;
        File xmlFile = new File(".." + File.separator + "mondial.xml");
        try {
            // Per validar un fitxer en el procés de càrrega, cal utilitzar un SAXBuilder amb instruccions
            // de validació, però si el fitxer a carregar no conté DOCTYPE, es produirà una excepció en la
            // validació. Per això creem el SAXBuilder amb validació o sense, segons el fitxer XML contingui
            // o no la cadena DOCTYPE i per saber-ho ens ajudem de FileUtils d'Apache Commons
            if (FileUtils.readFileToString(xmlFile, (String) null).contains("<!DOCTYPE")) {
                builder = new SAXBuilder(XMLReaders.DTDVALIDATING);
            } else {
                builder = new SAXBuilder();
            }
        } catch (IOException ex) {
            System.out.println("Error en comprovar si el fitxer XML conté etiqueta DOCTYPE");
            System.out.println("Excepció: " + ex.getClass().getName());
            System.out.println(ex.getMessage());
            System.exit(1);
        }
        try {
//            builder.setIgnoringElementContentWhitespace(true);  // Ignora elements que només continguin espais en blanc
            Document document = builder.build(xmlFile);
            Element rootNode = document.getRootElement();

            long habitants = 0;
            List<Element> listCountry = rootNode.getChildren("country");
            int nreCountry = listCountry.size();
            for (int i = 0; i < nreCountry; i++) {
                Element country = listCountry.get(i);
                /* Country */
                List<Element> listEncompassed = country.getChildren("encompassed");
                /* Conté atribut amb el nom del continent */
                int nreEncompassed = listEncompassed.size();
                for (int j = 0; j < nreEncompassed; j++) {
                    Element encompassed = listEncompassed.get(j);
                    // El DTD de l'arxiu Mondial que estem gestionant indica per country
                    // element population+ => seleccionarem la population d'any més actual
                    // i si d'un mateix any hi ha diverrses mesures (measeured) ens quedarem amb la "census"
                    if ("europe".equals(encompassed.getAttributeValue("continent"))) 
                    {
                        int any = -1;
                        long tmpHabitants = 0;
                        Float percentage = Float.parseFloat(encompassed.getAttributeValue("percentage"));
                        List<Element> llPopulation = country.getChildren("population");
                        for (int k = 0; k < llPopulation.size(); k++) {
                            Element population = llPopulation.get(k);
                            // Segons DTD, l'atribut "year" no és obligatori, però segons XSD, sí que ho és...
                            // Ens creiem el XDS... i no tenim en compte que "year" pugui ser null...
                            int aux = Integer.parseInt(population.getAttributeValue("year"));
                            if (aux > any || (aux == any && "census".equals(population.getAttributeValue("measured")))) {
                                any = aux;
                                tmpHabitants = Math.round(Long.parseLong(population.getValue().trim()) * percentage / 100.);
                            }
                        }
                        habitants = habitants + tmpHabitants;
                        break;
// Una vegada trobat continent "europe" dins els at. "encompassed" d'un "country", podríem "tallar" la comprovació
// dels següents "encompassed" doncs per lògica dins un "country" sembla que no hi hauria d'haver
                    }
                }
            }
            System.out.printf("Habitants: %,d\n", habitants);
        } catch (IOException io) {
            System.out.println("Error de IOException");
            System.out.println(io.getMessage());
        } catch (JDOMException jdomex) {
            System.out.println("Error de JDom");
            System.out.println(jdomex.getMessage());
        }
    }
}
