/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.milaifontanals.persistence;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import org.milaifontanals.Electrodomestic;
import org.milaifontanals.GammaBlanca;
import org.milaifontanals.GammaGris;

/**
 *
 * @author Usuari
 */
public class ElectrodomesticFitxerBinariFormatManual {

    private ElectrodomesticFitxerBinariFormatManual() {
    }

    /**
     * Crear un fitxer que contingui la llista d'electrodomèstics
     *
     * @param ll llista d'electrodomèstics a fer persistents
     * @param nomFitxer fitxer on enregistrar els electrodomèstics
     * (sobreescriure'l si existeix)
     */
    public static void saveListElectrodomestic(List<Electrodomestic> ll, String nomFitxer) {
        DataOutputStream dos = null;
        try {
            dos = new DataOutputStream(new FileOutputStream(nomFitxer));
            if (Electrodomestic.getDteVigent() != null) {
                dos.writeChar('S');
                dos.writeFloat(Electrodomestic.getDteVigent());
            } else {
                dos.writeChar('N');
            }
            for (Electrodomestic el : ll) {
                // Alerta... Una List podria contenir posicions amb valor NULL.
                if (el == null) {
                    continue;   // Passa al següent
                }
                if (el.getClass().getSimpleName().equals("GammaBlanca")) {
                    saveGammaBlanca((GammaBlanca) el, dos);
                } else if (el.getClass().getSimpleName().equals("GammaGris")) {
                    saveGammaGris((GammaGris) el, dos);
                } else {
                    throw new ElectrodomesticFitxerBinariFormatManualException("Objecte de classe "
                            + el.getClass().getSimpleName() + " no previst en aquesta capa de persistència");
                }
            }
        } catch (FileNotFoundException ex) {
            throw new ElectrodomesticFitxerBinariFormatManualException("Error en crear arxiu " + nomFitxer, ex);
        } catch (IOException ex) {
            throw new ElectrodomesticFitxerBinariFormatManualException("Error en enregistrar dada", ex);
        } finally {
            if (dos != null) {
                try {
                    dos.close();
                } catch (IOException ex) {
                    throw new ElectrodomesticFitxerBinariFormatManualException("Error en tancar arxiu " + nomFitxer, ex);
                }
            }
        }
    }

    private static void saveGammaBlanca(GammaBlanca el, DataOutputStream dos) throws IOException {
        dos.writeChar('B');     // Marca de GammaBlanca
        saveElectrodomestic(el, dos);

//        double t[] = el.getMides();
//        dos.writeDouble(t[0]);
//        dos.writeDouble(t[1]);
//        dos.writeDouble(t[2]);
//        Alternativament
        dos.writeDouble(el.getAltura());
        dos.writeDouble(el.getAmplada());
        dos.writeDouble(el.getFondaria());
    }

    private static void saveGammaGris(GammaGris el, DataOutputStream dos) throws IOException {
        dos.writeChar('G');     // Marca de GammaGris
        saveElectrodomestic(el, dos);
        dos.writeUTF(el.getModel());
        if (el.getEfEnergia() == null) {
            dos.writeChar('N');     // No hi ha valor (és null)
        } else {
            dos.writeChar('S');
            dos.writeUTF(el.getEfEnergia());
        }
        if (el.getPolsades() == null) {
            dos.writeChar('N');
        } else {
            dos.writeChar('S');
            dos.writeInt(el.getPolsades());
        }
    }

    private static void saveElectrodomestic(Electrodomestic el, DataOutputStream dos) throws IOException {
        dos.writeUTF(el.getNom());
        dos.writeUTF(el.getMarca());
        dos.writeDouble(el.getPreu());
        if (el.getGarantia() == null) {
            dos.writeChar('N');
        } else {
            dos.writeChar('S');
            dos.writeByte(el.getGarantia());
        }
        if (el.getDataPatent() == null) {
            dos.writeChar('N');
        } else {
            dos.writeChar('S');
            dos.writeLong(el.getDataPatent().getTimeInMillis());
        }
    }

    public static List<Electrodomestic> loadListElectrodomestic(String nomFitxer) {
        DataInputStream dis = null;
        List<Electrodomestic> ll = new ArrayList();
        try {
            dis = new DataInputStream(new FileInputStream(nomFitxer));
            if (dis.readChar() == 'S') {
                Electrodomestic.setDteVigent(dis.readFloat());
            } else {
                Electrodomestic.setDteVigent(null);
            }
            while (true) {
                char marca = dis.readChar();
                Electrodomestic el;
                if (marca == 'B' || marca == 'G') {
                    el = loadElectrodomestic(dis, marca);
                } else {
                    throw new ElectrodomesticFitxerBinariFormatManualException("S''ha trobat marca "
                            + marca + " d'objecte no prevista, diferent de B o G, dins el fitxer");
                }
                ll.add(el);
            }
        } catch (EOFException ex) {
            // No passa res... hem arribat al final del fitxer en intentar llegir una marca
        } catch (FileNotFoundException ex) {
            throw new ElectrodomesticFitxerBinariFormatManualException("Error en obrir arxiu " + nomFitxer, ex);
        } catch (IOException ex) {
            throw new ElectrodomesticFitxerBinariFormatManualException("Error en recuperar dada", ex);
        } finally {
            if (dis != null) {
                try {
                    dis.close();
                } catch (IOException ex) {
                    throw new ElectrodomesticFitxerBinariFormatManualException("Error en tancar arxiu " + nomFitxer, ex);
                }
            }
        }
        return ll;
    }

    private static Electrodomestic loadElectrodomestic(DataInputStream dis, char queEs) throws IOException {
        String nom = dis.readUTF();
        String marca = dis.readUTF();
        double preu = dis.readDouble();
        // Cal veure si constava garantia
        Byte garantia = null;
        char aux = dis.readChar();
        if (aux == 'S') {
            garantia = dis.readByte();
        }
        // Cal veure si constava dataPatent
        GregorianCalendar dataPatent = null;
        aux = dis.readChar();
        if (aux == 'S') {
            dataPatent = new GregorianCalendar();
            dataPatent.setTimeInMillis(dis.readLong());
        }

        if (queEs == 'B') {
            return new GammaBlanca(nom, marca, preu, garantia, dataPatent,
                    dis.readDouble(), dis.readDouble(), dis.readDouble());
        } else {
            String model = dis.readUTF();
            // Cal veure si constava o no efEnergia
            String efEnergia = null;
            aux = dis.readChar();
            if (aux == 'S') {
                efEnergia = dis.readUTF();
            }
            // Cal veure si constava o no polsades
            Integer polsades = null;
            aux = dis.readChar();
            if (aux == 'S') {
                polsades = dis.readInt();
            }
            return new GammaGris(nom, marca, preu, garantia, dataPatent, model, efEnergia, polsades);
        }
    }
}
