/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.milaifontanals.persistence;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;
import org.apache.commons.io.FileUtils;
import org.jdom2.DocType;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.jdom2.input.sax.XMLReaders;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;
import org.milaifontanals.Electrodomestic;
import org.milaifontanals.GammaBlanca;
import org.milaifontanals.GammaGris;

/**
 *
 * @author Usuari
 */
public class ElectrodomesticFitxerXmlV1ViaJDom {

    /**
     * Crear un fitxer que contingui la llista d'electrodomèstics
     *
     * @param ll llista d'electrodomèstics a fer persistents
     * @param nomFitxer fitxer on enregistrar els electrodomèstics
     * (sobreescriure'l si existeix)
     */
    private static DecimalFormat df = new DecimalFormat();
    private static SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy");

    static {
        DecimalFormatSymbols dfs = new DecimalFormatSymbols();
        dfs.setDecimalSeparator(',');
        df.setDecimalFormatSymbols(dfs);
    }

    public static void saveListElectrodomestic(List<Electrodomestic> ll, String nomFitxer) {
        Element arrel = new Element("listHomeAppliance");
        arrel.setAttribute("currentDiscount", df.format(Electrodomestic.getDteVigent()));
        Document doc = new Document(arrel);
        for (Electrodomestic el : ll) {
            // Alerta... Una List podria contenir posicions amb valor NULL.
            if (el == null) {
                continue;   // Passa al següent
            }
            if (el.getClass().getSimpleName().equals("GammaBlanca")) {
                arrel.addContent(gammaBlancaToElement((GammaBlanca) el));
            } else if (el.getClass().getSimpleName().equals("GammaGris")) {
                arrel.addContent(gammaGrisToElement((GammaGris) el));
            } else {
                throw new ElectrodomesticFitxerXmlViaV1JDomException("Mètode saveListElectrodomestic no preparat per "
                        + "enregristrar objecte de la classe "
                        + el.getClass().getSimpleName());
            }
        }
        // Inserim etiqueta <!DOCTYPE> al document
        doc.setDocType(new DocType("listHomeAppliance", "listHomeApplianceV1.dtd"));
        // Procés d'enregistrament a disc
        XMLOutputter out = new XMLOutputter();
        out.setFormat(Format.getPrettyFormat());
        try {
            out.output(doc, new FileWriter(nomFitxer));
        } catch (IOException ex) {
            throw new ElectrodomesticFitxerXmlViaV1JDomException("Error en enregistrar fitxer "
                    + nomFitxer, ex);
        }
    }

    private static Element gammaBlancaToElement(GammaBlanca gb) {
        Element el = electrodomesticToElement(gb);
        el.setName("whiteRange");
        el.addContent(new Element("height").setText(df.format(gb.getAltura())));
        el.addContent(new Element("width").setText(df.format(gb.getAmplada())));
        el.addContent(new Element("depth").setText(df.format(gb.getFondaria())));
        return el;
    }

    private static Element gammaGrisToElement(GammaGris gg) {
        Element el = electrodomesticToElement(gg);
        el.setName("grayRange");
        el.addContent(new Element("model").setText("" + gg.getModel()));
        if (gg.getEfEnergia() != null) {
            el.addContent(new Element("energyEfficiency").setText(gg.getEfEnergia()));
        }
        if (gg.getPolsades() != null) {
            el.addContent(new Element("inches").setText("" + gg.getPolsades()));
        }

        return el;
    }

    private static Element electrodomesticToElement(Electrodomestic obj) {
        Element el = new Element("temporal");
        el.setAttribute("code", "HA-" + obj.getId());
        el.addContent(new Element("name").setText(obj.getNom()));
        el.addContent(new Element("brand").setText(obj.getMarca()));
        el.addContent(new Element("price").setText(df.format(obj.getPreu())));
        if (obj.getGarantia() != null) {
            el.addContent(new Element("warranty").setText(obj.getGarantia().toString()));
        }
        if (obj.getDataPatent() != null) {
            el.addContent(new Element("patentDate").setText(sdf.format(obj.getDataPatent().getTime())));
        }
        return el;
    }

    public static List<Electrodomestic> loadListElectrodomestic(String nomFitxer) {
        SAXBuilder builder = null;
        File file = new File(nomFitxer);
        try {
            if (FileUtils.readFileToString(file, (String) null).contains("<!DOCTYPE")) {
                builder = new SAXBuilder(XMLReaders.DTDVALIDATING);
            } else {
                builder = new SAXBuilder();
            }
        } catch (IOException ex) {
            throw new ElectrodomesticFitxerXmlViaV1JDomException("Error en comprovar si el fitxer XML conté etiqueta DOCTYPE", ex);
        }
        Document doc = null;
        try {
            doc = builder.build(file);
        } catch (JDOMException | IOException ex) {
            throw new ElectrodomesticFitxerXmlViaV1JDomException("Error en carregar document XML en memòria", ex);
        }

        List<Electrodomestic> ll = new ArrayList();

        // Podem suposar que el contingut del fitxer XML és correcte
        // i no cal anar comprovant cada dada. Però posarem tota la càrrega
        // dins un try... catch... per si hi hagués algun problema de contingut erroni
        try {
            Element arrel = doc.getRootElement();
            Electrodomestic.setDteVigent(df.parse(arrel.getAttributeValue("currentDiscount")).floatValue());

            List<Element> electrodomestics = arrel.getChildren();
            for (Element el : electrodomestics) {
                String tipus = el.getName();
                if (tipus.equals("whiteRange")) {
                    ll.add(elementToGammaBlanca(el));
                } else if (tipus.equals("grayRange")) {
                    ll.add(elementToGammaGris(el));
                } else {
                    throw new ElectrodomesticFitxerXmlViaV1JDomException("Mètode loadListElectrodomestic no preparat per "
                            + "carregar element Electrodomestic amb nom " + tipus);
                }
            }
        } catch (Exception ex) {
            throw new ElectrodomesticFitxerXmlViaV1JDomException("Error: " + ex.getMessage(), ex.getCause());
        }
        return ll;
    }

    private static GammaBlanca elementToGammaBlanca(Element el) throws ParseException {
        GammaBlanca gb = (GammaBlanca) elementToElectrodomestic(el, 'B');
        gb.setAltura(df.parse(el.getChildText("height")).doubleValue());
        gb.setAmplada(df.parse(el.getChildText("width")).doubleValue());
        gb.setFondaria(df.parse(el.getChildText("depth")).doubleValue());
        return gb;
    }

    private static GammaGris elementToGammaGris(Element el) throws ParseException {
        GammaGris gg = (GammaGris) elementToElectrodomestic(el, 'G');
        gg.setModel(el.getChildText("model"));

        String efEnergia = el.getChildText("energyEfficiency");
        if ("".equals(efEnergia)) {
            efEnergia = null;
        }
        gg.setEfEnergia(efEnergia);

        String auxPolsades = el.getChildText("inches");
        Integer polsades = null;
        if (auxPolsades != null && !"".equals(auxPolsades)) {
            polsades = Integer.parseInt(auxPolsades);
        }
        gg.setPolsades(polsades);

        return gg;
    }

    private static Electrodomestic elementToElectrodomestic(Element el, char queEs) throws ParseException {
        int id = Integer.parseInt(el.getAttributeValue("code").substring(3));
        String nom = el.getChildText("name");
        String marca = el.getChildText("brand");
        double preu = df.parse(el.getChildText("price")).doubleValue();

        Byte garantia = null;
        String auxGarantia = el.getChildText("warranty");
        if (auxGarantia != null && !"".equals(auxGarantia)) {
            garantia = Byte.parseByte(auxGarantia);
        }
        GregorianCalendar dataPatent = null;
        String auxDataPatent = el.getChildText("patentDate");

        if (auxDataPatent != null && !"".equals(auxDataPatent)) {
            dataPatent = new GregorianCalendar();
            dataPatent.setTime(sdf.parse(auxDataPatent));
        }
        Electrodomestic obj;

        if (queEs == 'B') {
            obj = new GammaBlanca(id, nom, marca, preu, new double[]{1, 1, 1});
            // Per poder crear l'electrodomestic de GammaBlanca ens hem vist obligats
            // a crear-lo amb unes mides falses, que modificarem posteriorment.
        } else {
            obj = new GammaGris(id, nom, marca, preu, "???");
            // Per poder crear l'electrodomestic de GammaGris ens hem vist obligats
            // a crear-lo amb model fals, que modificarem posteriorment.
        }
        obj.setGarantia(garantia);
        obj.setDataPatent(dataPatent);

        return obj;
    }

}
