/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.milaifontanals.persistence;

import java.io.File;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.List;
import jakarta.xml.bind.JAXBContext;
import jakarta.xml.bind.Marshaller;
import jakarta.xml.bind.Unmarshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.apache.commons.io.FileUtils;
import org.milaifontanals.Electrodomestic;
import org.milaifontanals.GammaBlanca;
import org.milaifontanals.GammaGris;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 *
 * @author Usuari
 */
public class ElectrodomesticFitxerXmlV2ViaJaxb {

    /**
     * Crear un fitxer que contingui la llista d'electrodomèstics
     *
     * @param ll llista d'electrodomèstics a fer persistents
     * @param nomFitxer fitxer on enregistrar els electrodomèstics
     * (sobreescriure'l si existeix)
     */
    private static DecimalFormat df = new DecimalFormat();

    static {
        DecimalFormatSymbols dfs = new DecimalFormatSymbols();
        dfs.setDecimalSeparator(',');
        df.setDecimalFormatSymbols(dfs);
    }

    public static void saveListElectrodomestic(List<Electrodomestic> ll, String nomFitxer) {
        try {
            // Procés estàndard via org.w3c.dom per crear document XML nou (M03-UF5)
            DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder dbuilder = dbFactory.newDocumentBuilder();
            Document doc = dbuilder.newDocument();
            Element arrel = doc.createElement("listHomeAppliance");
            arrel.setAttribute("currentDiscount", df.format(Electrodomestic.getDteVigent()));
            doc.appendChild(arrel);
            // Fi procés creació capçalera document - Encara no hem aplicat JAXB

            // Per abocar dins el document els objectes Electrodomestic, utilitzem JAXB
            JAXBContext jaxbContext = JAXBContext.newInstance(GammaBlanca.class, GammaGris.class);
            Marshaller jaxbMarshaller = jaxbContext.createMarshaller();

            jaxbMarshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);

            for (Electrodomestic e : ll) {
                if (e != null) {
                    jaxbMarshaller.marshal(e, arrel);
                } else {
                    throw new ElectrodomesticFitxerXmlV2ViaJaxbException("Intent d'enregistrar llista amb algun valor null");
                }
            }
            // S'acaba l'abocament d'objectes via JAXB

            // Enregistrar el contingut a disc, via procés estàndard (M03-UF5)
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(doc);
            StreamResult result = new StreamResult(new File(nomFitxer));
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "2");
            transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
            transformer.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, "listHomeApplianceV2.dtd");
            transformer.transform(source, result);
        } catch (Exception ex) {
            throw new ElectrodomesticFitxerXmlV2ViaJaxbException("Error en mètode saveListElectrodomestic", ex);
        }
    }

    public static List<Electrodomestic> loadListElectrodomestic(String nomFitxer) {
        try {
            List<Electrodomestic> ll = new ArrayList();

            // Procés estàndard via org.w3c.dom per recuperar document XML (M03-UF5)
            DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
            dbFactory.setValidating(true);
            DocumentBuilder db = dbFactory.newDocumentBuilder();
            if (!FileUtils.readFileToString(new File(nomFitxer), (String) null).contains("<!DOCTYPE")) {
                db.setErrorHandler(null);   // Desactiva la validació
            } else {
                db.setErrorHandler(new ErrorHandler() {
                    @Override
                    public void warning(SAXParseException ex) throws SAXException {
                        throw new ElectrodomesticFitxerXmlV2ViaJaxbException("Error en validar fitxer " + nomFitxer, ex);
                    }

                    @Override
                    public void error(SAXParseException ex) throws SAXException {
                        throw new ElectrodomesticFitxerXmlV2ViaJaxbException("Error en validar fitxer " + nomFitxer, ex);
                    }

                    @Override
                    public void fatalError(SAXParseException ex) throws SAXException {
                        throw new ElectrodomesticFitxerXmlV2ViaJaxbException("Error en validar fitxer " + nomFitxer, ex);
                    }
                });
            }
            Document doc = db.parse(new File(nomFitxer));
            Element arrel = doc.getDocumentElement();
            arrel.normalize();
            // Gestió estàndard

            // JAXB per recuperar tots els elements
            JAXBContext jbBlanca = JAXBContext.newInstance(GammaBlanca.class);
            Unmarshaller unBlanca = jbBlanca.createUnmarshaller();
            JAXBContext jbGris = JAXBContext.newInstance(GammaGris.class);
            Unmarshaller unGris = jbGris.createUnmarshaller();

            NodeList llista = arrel.getElementsByTagName("homeAppliance");
            int q = llista.getLength();
            int i;
            for (i = 0; i < q; i++) {
                Electrodomestic e;

                // Hem d'esbrinar, abans d'invocar l'Unmarshaller, de quin tipus d'objecte es tracta
                Element aux = (Element)llista.item(i);
                if (aux.getElementsByTagName("whiteRange").getLength()==1) {
                    e = (GammaBlanca) unBlanca.unmarshal(aux);
                    System.out.println(e);
                } else if (aux.getElementsByTagName("grayRange").getLength()==1) {
                    e = (GammaGris) unGris.unmarshal(aux);
                    System.out.println(e);
                } else {
                    throw new ElectrodomesticFitxerXmlV2ViaJaxbException("Mètode load no preparat per "
                            + "carregar element que no és whiteRange ni grayRange");
                }
                ll.add(e);
            }

            String sCurrentDiscount = arrel.getAttribute("currentDiscount");
            Electrodomestic.setDteVigent(df.parse(sCurrentDiscount).floatValue());

            return ll;
        } catch (Exception ex) {
            throw new ElectrodomesticFitxerXmlV2ViaJaxbException("Error en mètode loadListElectrodomestic", ex);
        }
    }

}
